
package org.ow2.petals.bc.jms.connection;

import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.spi.InitialContextFactory;

import org.apache.commons.pool.BasePoolableObjectFactory;
import org.ow2.petals.component.framework.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.exception.TechnicalException;
import org.ow2.petals.component.framework.util.Extensions;

public abstract class AbstractJMSConnectionFactory extends BasePoolableObjectFactory {

    private static final String CONNECTION_FACTORY = "connection-factory";

    private static final String ADDRESS = "address";

    private static final String DESTINATION_NAME = "destination-name";

    private static final String INITIAL_CONTEXT_FACTORY = "initial-context-factory";

    private static final String USER = "user";

    private static final String PASSWORD = "password";

    private static final String TRANSACTED = "transacted";

    private Extensions extensions;

    private Context context;

    private Logger logger;

    public AbstractJMSConnectionFactory(Extensions extensions, String suRootPath, Logger logger)
            throws TechnicalException {
        super();
        this.logger = logger;
        this.extensions = extensions;

        // Create the JNDI context used to retrieve jms connection and
        // destination
        this.context = setupContext(suRootPath, extensions);
    }

    @Override
    public void activateObject(Object arg0) throws Exception {
        AbstractJMSConnection connection = (AbstractJMSConnection) arg0;
        connection.start();
        logger.log(Level.FINEST, "activate connection");
    }

    @Override
    public void passivateObject(Object arg0) throws Exception {
        AbstractJMSConnection connection = (AbstractJMSConnection) arg0;
        connection.stop();
        logger.log(Level.FINEST, "passivate connection");
    }

    @Override
    public Object makeObject() throws Exception {
        logger.log(Level.FINEST, "create connection");
        // create the jmsConnection object that contains all information
        AbstractJMSConnection connection = createJmsConnection(context, extensions);

        connection.init();
        return connection;
    }

    /**
     * Setup a JNDIContext to access a JMS destination.
     *
     * @param suRootPath
     *            path where jndi context factory class can be found
     * @param extensions
     *            jndi properties
     * @return a JNDI context initialized with the given jbi.xml properties
     * @throws TechnicalException
     *             if JNDI context cannot be created
     */
    private Context setupContext(String suRootPath, Extensions extensions)
            throws TechnicalException {

        // Create JNDI properties
        Properties jndiProperties = createJNDIProperties(extensions);

        // Create the JNDI context
        return createJNDIContext(jndiProperties);
    }

    /**
     * extract jndi properties from the jbi.xml extensions
     *
     * @param extensions
     * @return
     */
    private Properties createJNDIProperties(Extensions extensions) {
        Properties outProperties = new Properties();

        String url = extensions.getValue(ADDRESS);
        String factory = extensions.getValue(INITIAL_CONTEXT_FACTORY);

        outProperties.setProperty(InitialContext.PROVIDER_URL, url);
        outProperties.setProperty(InitialContext.INITIAL_CONTEXT_FACTORY, factory);
        return outProperties;
    }

    /**
     * Return a JNDI context loaded from the JAR files contained in the
     * ServiceUnit and the Properties defined in the JBI.XML file.
     *
     * @param properties
     *            contains the factory class name and the url provider
     * @return a JNDI context
     * @throws TechnicalException
     *             can not retrieved the factory. properties might not be
     *             correct.
     *
     */
    private Context createJNDIContext(Properties properties) throws TechnicalException {
        Context ctx = null;
        try {
            ClassLoader cl = this.getClass().getClassLoader();
            Class clazz = cl.loadClass(properties
                    .getProperty(InitialContext.INITIAL_CONTEXT_FACTORY));
            InitialContextFactory icf = (InitialContextFactory) clazz.newInstance();
            ctx = icf.getInitialContext(properties);
        } catch (Exception e) {
            throw new TechnicalException("JNDI context can not be created." + e, e);
        }
        return ctx;
    }

    /**
     * Setup a JMS connection to access a JMS destination.
     *
     * @param context
     *            jndi context where the connection factory and the destination
     *            can be found
     * @param extensions
     *            contains connectionFactory and destination names, user/pwd,
     *            transacted and acknowledge modes.
     * @return a JMSConnection in a stopped state.
     * @throws PEtALSCFException
     *             a mandatory property is not found in the JBI.XML descriptor
     */
    private AbstractJMSConnection createJmsConnection(Context context, Extensions extensions)
            throws PEtALSCDKException{
        String connName = extensions.getValue(CONNECTION_FACTORY);

        String destName = extensions.getValue(DESTINATION_NAME);

        String user = extensions.getValue(USER);

        String password = extensions.getValue(PASSWORD);

        boolean transacted = false;
        String transac = extensions.getValue(TRANSACTED);
        transacted = Boolean.valueOf(transac);

        return createJMSConnection(context, connName, destName, user, password, transacted);

    }

    @Override
    public void destroyObject(Object obj) throws Exception {
        AbstractJMSConnection connection = (AbstractJMSConnection) obj;
        connection.close();
        logger.log(Level.FINEST, "destroy connection");
    }

    public abstract AbstractJMSConnection createJMSConnection(Context context, String connName,
            String destName, String user, String password, boolean transacted);

}
