
package org.ow2.petals.bc.jms.listener;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.logging.Level;

import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jms.JMSException;
import javax.jms.JMSSecurityException;
import javax.jms.TextMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.pool.impl.GenericObjectPool;
import org.ow2.petals.bc.jms.JmsBC;
import org.ow2.petals.bc.jms.connection.JMSProviderConnection;
import org.ow2.petals.component.framework.Constants.MEPConstants;
import org.ow2.petals.component.framework.exception.TechnicalException;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.Exchange;

public class ServiceManager {

    private static final String JMS_OPERATION_PROPERTY = "operation";

    private AbstractJBIListener listener;

    public ServiceManager(AbstractJBIListener listener) {
        super();
        this.listener = listener;
    }

    public void onJBIMessage(Exchange exchange) throws TechnicalException {
        if (MEPConstants.IN_ONLY_PATTERN.value().equals(exchange.getPattern())
                || MEPConstants.ROBUST_IN_ONLY_PATTERN.value().equals(exchange.getPattern())
                || MEPConstants.IN_OPTIONAL_OUT_PATTERN.value().equals(exchange.getPattern())) {
            sendJMSMessage(exchange);
        } else if (MEPConstants.IN_OUT_PATTERN.value().equals(exchange.getPattern())) {
            sendJMSMessage(exchange);
            try {
                exchange.setOutMessageContent("<content>JMSMessageSent</content>");
            } catch (MessagingException e) {
                throw new TechnicalException("Failed to set out message content", e);
            }
        } else {
            throw new TechnicalException("MessageExchangePattern not allowed :"
                    + exchange.getPattern());
        }
    }

    /**
     * Send a JMS message to the JMS queue corresponding to the specified
     * exchange's endpoint.
     *
     * @param exchange
     * @throws TechnicalException
     *             no JMS destination found, or problem while sending the
     *             message to JMS
     *
     */
    protected void sendJMSMessage(Exchange exchange) throws TechnicalException {
        listener.getLogger().log(Level.FINE, "sendJMSMessage");

        // find the connection to use for this exchange
        JmsBC component = (JmsBC) listener.getComponent();
        GenericObjectPool pool = component.getJmsProducerConnections().get(listener.getProvides());
        JMSProviderConnection conn;
        try {
            conn = (JMSProviderConnection) pool.borrowObject();
        } catch (Exception e1) {
            String exceptionMsg = "Can't retrieve a JMS connection "
                    + "linked to the following endpoint: " + exchange.getEndpoint();
            throw new TechnicalException(exceptionMsg, e1);
        }
        try {
            if (conn == null) {
                // No such endpoint registered
                throw new TechnicalException("Can't retrieve a JMS connection "
                        + "linked to the following endpoint: " + exchange.getEndpoint());
            } else {
                listener.getLogger().log(
                        Level.INFO,
                        "Try to send JBI message to '" + conn.getDestinationName()
                                + "' JMS provider.");
            }

            // transform the jbi msg into a jms msg
            TextMessage msg = null;
            try {
                msg = conn.getSession().createTextMessage();
                transform(exchange, msg);
            } catch (Exception e) {
                listener.getLogger().log(Level.SEVERE,
                        "Can not create the JMS text message from JBI message.", e);
                throw new TechnicalException(
                        "Can not create the JMS text message from JBI message.", e);
            }

            // send
            try {
                conn.getMessageProducer().send(msg);
                if (conn.isTransacted()) {
                    conn.getSession().commit();
                }
            } catch (JMSSecurityException e) {
                String exceptionMsg = "Can not send the JMS message, WRITE right not granted.";
                throw new TechnicalException(exceptionMsg, e);
            } catch (JMSException e) {
                String exceptionMsg = "Can not send the JMS message.";
                throw new TechnicalException(exceptionMsg, e);
            }
        } finally {
            try {
                pool.returnObject(conn);
            } catch (Exception e) {
                String exceptionMsg = "Can't free a JMS connection of the pool.";
                throw new TechnicalException(exceptionMsg, e);
            }
        }
        listener.getLogger()
                .log(Level.INFO, "Message sent to '" + conn.getDestinationName() + "'.");
    }

    /**
     * Transform the NormalizedMessage of a JBI MsgExch into a TextMessage. The
     * content , properties and attachments of the IN NormalizedMessage are set.
     *
     * @param exchange
     * @param jms
     * @throws JMSException
     *             writing properties to the TextMessage
     * @throws TransformerException
     *             NormalizedMessage content access error
     * @throws IOException
     *             NormalizedMessage content access error
     */
    protected void transform(Exchange exchange, TextMessage jms) throws JMSException,
            TransformerException, IOException {
        listener.getLogger().log(Level.FINE, "transform");

        NormalizedMessage in = exchange.getInMessage();

        // set normalized message properties
        for (Iterator<?> it = in.getPropertyNames().iterator(); it.hasNext();) {
            String pName = (String) it.next();
            Object pValue = in.getProperty(pName);
            jms.setObjectProperty(pName, pValue);
        }

        QName operation = exchange.getOperation();
        if (operation != null) {
            jms.setStringProperty(JMS_OPERATION_PROPERTY, operation.toString());
        }

        // set content
        StreamResult stream = new StreamResult(new ByteArrayOutputStream());
        Transformer transformer = TransformerFactory.newInstance().newTransformer();
        StreamSource source = (StreamSource) in.getContent();
        transformer.transform(source, stream);
        source.getInputStream().reset();
        stream.getOutputStream().close();
        jms.setText(stream.getOutputStream().toString());
    }
}
